{ DosEnv: Main Window }
unit DosEnv01;

interface

uses
  DosEnv00,
  ExtCtrls, Menus, Dialogs, StdCtrls, Controls, Buttons, Classes,
  Windows, Messages, SysUtils, Graphics, Forms, Grids;

type
  TMainWindow = class(TForm)
    ToolBar: TPanel;
    FontDialog: TFontDialog;
    MainMenu: TMainMenu;
    MnuFile: TMenuItem;
    MnuFileExit: TMenuItem;
    MnuEdit: TMenuItem;
    MnuEditCopy: TMenuItem;
    MnuOptions: TMenuItem;
    MnuOptToolBar: TMenuItem;
    MnuOptHints: TMenuItem;
    N1: TMenuItem;
    MnuOptSorted: TMenuItem;
    MnuOptFont: TMenuItem;
    MnuHelp: TMenuItem;
    MnuHelpAbout: TMenuItem;
    BtnExit: TSpeedButton;
    BtnCopy: TSpeedButton;
    BtnHints: TSpeedButton;
    BtnFont: TSpeedButton;
    BtnSorted: TSpeedButton;
    BtnHelp: TSpeedButton;
    StringGrid: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormResize(Sender: TObject);
    procedure StringGridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FontDialogApply(Sender: TObject; Wnd: Integer);
    procedure MnuFileExitClick(Sender: TObject);
    procedure MnuEditCopyClick(Sender: TObject);
    procedure MnuOptionsClick(Sender: TObject);
    procedure MnuOptToolBarClick(Sender: TObject);
    procedure MnuOptHintsClick(Sender: TObject);
    procedure MnuOptFontClick(Sender: TObject);
    procedure MnuOptSortedClick(Sender: TObject);
    procedure MnuHelpAboutClick(Sender: TObject);
  private
    Env       : TStringList;
    procedure WMGetMinMaxInfo(var V_Msg: Tmessage); message WM_GetMinMaxInfo;
    procedure GetEnv;
    procedure DisplayEnv;
  public
    OptSorted : boolean;
    procedure ApplyFont;
    procedure AdjustControls;
  end;

var
  MainWindow: TMainWindow;

implementation

uses
  DosEnv02,
  Clipbrd;

{$R *.DFM}

{ Private }

procedure TMainWindow.WMGetMinMaxInfo(var V_Msg: Tmessage);
begin with PMinMaxInfo(V_Msg.lparam)^.ptMinTrackSize do begin
  x:= 350; y:= 100;
end; end;

procedure TMainWindow.GetEnv;
var P: PChar;
begin
  Env.Clear; Env.Sorted:= OptSorted;
  P:= GetEnvironmentStrings;
  try
    while P^ <> #0 do begin
      Env.Add(StrPas(P));
      Inc(P,StrLen(P)+1);
    end;
  finally FreeEnvironmentStrings(P); end;
end;

procedure TMainWindow.DisplayEnv;
var i: integer;
begin with StringGrid do begin
  Cells[0,0]:= 'Name';
  Cells[1,0]:= 'Value';
  if Env.Count > 0 then RowCount:= Env.Count + 1;
  for i:= 0 to Env.Count-1 do begin
    Cells[0,i+1]:= Env.Names[i];
    Cells[1,i+1]:= Env.Values[Env.Names[i]];
  end;
end; end;

{ Public }

procedure TMainWindow.ApplyFont;
var h: integer;
begin with StringGrid do begin
  Font:= FontDialog.Font;
  if Font.Height < 0 then h:= abs(Font.Height) + 4
                     else h:= Font.Height;
  DefaultRowHeight:= h;
end; end;

procedure TMainWindow.AdjustControls;
begin
  BtnHints.Down  := ShowHint;
  BtnSorted.Down := OptSorted;
end;


{ Generated }

procedure TMainWindow.FormCreate(Sender: TObject);
begin
  Icon:= Application.Icon;
  ApplyFont;
  Env:= TStringList.Create;
  Env.Duplicates:= dupAccept;
end;

procedure TMainWindow.FormDestroy(Sender: TObject);
begin
  Env.Destroy;
end;

procedure TMainWindow.FormShow(Sender: TObject);
begin
  Application.Title  := GC_ProgTitle;
  MainWindow.Caption := GC_ProgTitle;
  GV_SettingFnm      := ChangeFileExt(Application.ExeName,GC_SettingExt);
  ReadOptions;
  GetEnv; DisplayEnv;
end;

procedure TMainWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose:= WriteOptions;
end;

procedure TMainWindow.FormResize(Sender: TObject);
begin with StringGrid do begin
  ColWidths[1]:= Width - ColWidths[0] - GridLineWidth;
end end;

procedure TMainWindow.StringGridMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FormResize(Sender);
end;

procedure TMainWindow.FontDialogApply(Sender: TObject; Wnd: Integer);
begin
  ApplyFont;
end;

procedure TMainWindow.MnuFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainWindow.MnuEditCopyClick(Sender: TObject);
var i: integer; Buf: array[0..255] of char;
begin
  i:= StringGrid.Selection.Top - 1;
  if (i >= 0) and (i <= Env.Count-1) then
    Clipboard.SetTextBuf(StrPCopy(Buf,Env.Strings[i]+#13#10));
end;

procedure TMainWindow.MnuOptionsClick(Sender: TObject);
begin
  MnuOptToolbar.Checked  := Toolbar.Visible;
  MnuOptHints.Enabled    := Toolbar.Visible;
  if MnuOptHints.Enabled then
    MnuOptHints.Checked  := ShowHint
  else
    MnuOptHints.Checked  := false;
  MnuOptSorted.Checked   := OptSorted;
end;

procedure TMainWindow.MnuOptToolBarClick(Sender: TObject);
begin
  Toolbar.Visible:= not Toolbar.Visible;
end;

procedure TMainWindow.MnuOptHintsClick(Sender: TObject);
begin
  ShowHint:= not ShowHint;
  AdjustControls;
end;

procedure TMainWindow.MnuOptFontClick(Sender: TObject);
begin
  if FontDialog.Execute then ApplyFont;
end;

procedure TMainWindow.MnuOptSortedClick(Sender: TObject);
begin
  OptSorted:= not OptSorted;
  AdjustControls;
  GetEnv; DisplayEnv;
end;

procedure TMainWindow.MnuHelpAboutClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

end.
